import { Injectable } from "@angular/core";
import { URLSearchParams } from '@angular/http';
// Services
import { HttpService } from "./http.service";
// Url's
import { GroupsUrls } from "../urls/group-urls.model";
var GroupsService = (function () {
    function GroupsService(httpService) {
        this.httpService = httpService;
    }
    /**
     * Создание чата (канала\группы)
     * @param {ChatCreateInfoModel} chatData Информация для создания чата
     * @returns {Promise<ChatInfoModel>}
     */
    GroupsService.prototype.create = function (chatData) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups";
            _this.httpService.post(url, chatData)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Получение списка чатов пользователя (каналов\групп)
     * @param {boolean} onlyMy Показывать только мои чаты
     * @param {string} lastChatGuid Идентфиикатор последнего чата
     * @param {string} stopChatGuid Идентификатор чата, до которого получить информацию
     * @param {any} query Дополнительные параметры
     * @returns {Promise<ResponseListModel<ChatInfoModel>>}
     */
    GroupsService.prototype.getList = function (onlyMy, lastChatGuid, stopChatGuid, query) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/list";
            if (lastChatGuid) {
                url += "/" + lastChatGuid;
                if (stopChatGuid) {
                    url += "/" + stopChatGuid;
                }
            }
            if (onlyMy) {
                url += "?only-my";
            }
            var params = new URLSearchParams();
            if (query) {
                var keys = Object.keys(query);
                for (var i = 0; i < keys.length; i++) {
                    var key = keys[i];
                    params.set(key, query[key]);
                }
            }
            _this.httpService.get(url, { params: params })
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Просмотр информации о чате (канала группы)
     * @param {string} chatGuid Идентфикатор чата (канала\группы)
     * @returns {Promise<ChatInfoModel>}
     */
    GroupsService.prototype.getInfo = function (chatGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + chatGuid;
            _this.httpService.get(url)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Обновление информации о чате (канале\группе)
     * @param {string} chatGuid Идентификатор чата (канала\группы)
     * @param {ChatCreateInfoModel} params Параметры для обновления чата (канала\группы)
     * @returns {Promise<ChatInfoModel>}
     */
    GroupsService.prototype.update = function (chatGuid, params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + chatGuid;
            _this.httpService.put(url, params)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Удаление чата (канала\группы)
     * @param {string} chatGuid Идентификатор чата (канала\группы)
     * @returns {Promise<boolean>}
     */
    GroupsService.prototype.remove = function (chatGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + chatGuid;
            _this.httpService.delete(url)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Присоединиться к публичному чату (канала\группы)
     * @param {string} chatGuid Идентификатор чата (канала\группы)
     * @returns {Promise<boolean>}
     */
    GroupsService.prototype.join = function (chatGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + chatGuid + "/join";
            _this.httpService.post(url, {})
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Выход из чата (канала\группы)
     * @param {string} chatGuid Идентификатор чата (канала\группы)
     * @returns {Promise<boolean>}
     */
    GroupsService.prototype.leave = function (chatGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + chatGuid + "/leave";
            _this.httpService.delete(url, {})
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Принять приглашение на вступление в чат
     * @param {string} inviteGuid Идентификатор приглашения
     */
    GroupsService.prototype.accept = function (inviteGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = GroupsUrls.groupInvite + "/" + inviteGuid;
            _this.httpService.post(url, {})
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Отклонить приглашение на вступление в чат
     * @param {string} inviteGuid Идентификатор приглашения
     */
    GroupsService.prototype.refuse = function (inviteGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = GroupsUrls.groupInvite + "/" + inviteGuid;
            _this.httpService.delete(url, {})
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Приглашение пользователя в группу
     * @param {string} groupGuid Идентификатор группы
     * @param {string} userGuid Идентификатор пользователя
     * @returns {Promise<MemberInfoModel>}
     */
    GroupsService.prototype.invite = function (groupGuid, userGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + groupGuid + "/invite/" + userGuid;
            _this.httpService.post(url, null)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Поиск сообщений в чате
     * @param {string} chatGuid Идентификтаор чата
     * @param {string} query Строка поиска
     * @param {string} messageGuid Идентификатор сообщения до которого необходимо искать
     * @returns {Observable<MessageSearchResultModel>}
     */
    GroupsService.prototype.searchMessages = function (chatGuid, query, messageGuid) {
        var url = this.messagesEndPoint + "groups/" + chatGuid + "/search-messages";
        if (messageGuid) {
            url += '/' + messageGuid;
        }
        url += '?q=' + query;
        return this.httpService.get(url)
            .map(function (response) { return response.json(); })
            .map(function (response) { return response.data; });
    };
    /**
     * Поиск по чатам
     * @param {string} queryText Текст для поиска
     * @param {any} query Дополнительные параметры запроса
     */
    GroupsService.prototype.search = function (queryText, query) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/search";
            var params = new URLSearchParams();
            params.set('q', queryText);
            if (query) {
                var keys = Object.keys(query);
                for (var i = 0; i < keys.length; i++) {
                    var key = keys[i];
                    params.set(key, query[key]);
                }
            }
            return _this.httpService.get(url, { params: params })
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Получение истории сообщений
     * @param {string} chatGuid Идентификатор чата (канала\группы)
     * @param {string} messageGuid Идентификатор сообщения, от которого получать историю
     * @param {string} startMessageGuid Идентификатор сообщения, до которого получать историю
     * @param {number} numBefore Количество сообщений которые надо добавить до начального сообщения
     * @param {any} query Дополнительные параметры
     * @returns {Promise<ResponseListModel<ChatMessageModel>>}
     */
    GroupsService.prototype.getHistory = function (chatGuid, messageGuid, startMessageGuid, numBefore, query) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + chatGuid + "/history";
            if (messageGuid) {
                url += "/" + messageGuid;
                if (startMessageGuid) {
                    url += "/" + startMessageGuid;
                    if (numBefore) {
                        url += "/" + numBefore;
                    }
                }
            }
            var params = new URLSearchParams();
            if (query) {
                var keys = Object.keys(query);
                for (var i = 0; i < keys.length; i++) {
                    var key = keys[i];
                    params.set(key, query[key]);
                }
            }
            _this.httpService.get(url, { params: params })
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Получаем список участников группы
     *
     * @param {string} groupGuid  Индификатор группы
     * @param query               Дополнительные параметры
     */
    GroupsService.prototype.getMembers = function (groupGuid, query) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + groupGuid + "/members";
            var params = new URLSearchParams();
            if (query) {
                var keys = Object.keys(query);
                for (var i = 0; i < keys.length; i++) {
                    var key = keys[i];
                    params.set(key, query[key]);
                }
            }
            _this.httpService.get(url, { params: params })
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Поиск по участникам групп
     *
     * @param groupGuid     Индификатор группы в по списку участников которой производим поиск
     * @param queryText     Текст по которому ищем
     * @param query         Значение по которому производим поиск участников
     */
    GroupsService.prototype.searchMembers = function (groupGuid, queryText, query) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + groupGuid + "/members/search";
            var params = new URLSearchParams();
            params.set('q', queryText);
            if (query) {
                var keys = Object.keys(query);
                for (var i = 0; i < keys.length; i++) {
                    var key = keys[i];
                    params.set(key, query[key]);
                }
            }
            return _this.httpService.get(url, { params: params })
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Удаление участника группы
     *
     * @param {string} groupGuid
     * @param {string} userGuid
     * @returns {Promise<boolean>}
     */
    GroupsService.prototype.removeMember = function (groupGuid, userGuid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + groupGuid + "/kickoff/" + userGuid;
            _this.httpService.delete(url)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Получение информации о статусе приглашения для пользователей группы
     * @param  {string} groupGuid Идентификатор группы
     * @param  {string[]} userGuids Массив идентификаторов пользователей группы
     * @return {Promise<MemberInfoModel[]>}
     */
    GroupsService.prototype.getMembersInfo = function (groupGuid, userGuids) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var url = _this.messagesEndPoint + "groups/" + groupGuid + "/members-info";
            _this.httpService.post(url, userGuids)
                .map(function (response) { return response.json(); })
                .map(function (response) { return response.data; })
                .subscribe(resolve, reject);
        });
    };
    /**
     * Установка URL для сервиса сообщений
     * @param messagesEndPoint
     */
    GroupsService.prototype.setMessagesEndPoint = function (messagesEndPoint) {
        this.messagesEndPoint = messagesEndPoint;
    };
    return GroupsService;
}());
export { GroupsService };
GroupsService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
GroupsService.ctorParameters = function () { return [
    { type: HttpService, },
]; };
//# sourceMappingURL=groups.service.js.map